; Pill Box PillBox - January 24, 2011

;Modification:  January 24, 2011 ... add new clock correction routines (copied from Ultimate Night Light)
;  storage of crystal correction values.


;Wayne Simister
;Automated Sound
;584 Cortez Street
;Salt Lake City, UT 84103
;wsimister@hotmail.com


;r15 = seconds counter
;r26 = LSD of Total Count in Button 
;r27 = MSD of Total Count in Button

;r18 Flags

;r18,0(1) 2nd flash, RED led
;r18,1(2) 2nd flash, YELLOW led
;r18,2(4) 2nd flash, GREEN led
;r18,3(8) 2nd flash, BLUE led
;r18,4(16)
;r18,5(32)
;r18,6(64) Clock Correction - 0=yes = ON
;r18,7(128) Variable Day Clock Correction - add one second next time flag



;r19 Flags

;r19,0(1) RED led
;r19,1(2) YELLOW led
;r19,2(4) GREEN led
;r19,3(8) BLUE led
;r19,4(16) Temp LED flag
;r19,5(32) White led flash each minute
;r19,6(64) Minute flag
;r19,7(128) 



;ALL entries are a dit, dah, or hold down push of button.  Dit=1, Dah=10, Hold down= 50

;Dit 1 = Pill taken, toggle on/off 2nd pulse RED LED flash
;Dit 2 = Pill taken, toggle on/off 2nd pulse YELLOW LED flash
;DIT 3 = Pill taken, toggle on/off 2nd pulse GREEN LED flash
;DIT 4 = Pill taken, toggle on/off 2nd pulse BLUE LED flash

;Dit 11 = Program RED LED wth pattern (up to eight bits)
;Dit 12 = Program YELLOW LED with pattern
;Dit 13 = Program GREEN LED
;Dit 14 = Program BLUE LED



;NOTE: Some entries use a +1 entry method to allow an entry to handle entry of 1 = 0)


;Dit 16 = Clear all program codes for all LEDs
;Dit 17 = Toggle a flash of the white LED every minute - used for timing setup purposes
;Dit 18 = Set Clock (DOW,Hour,Min)(Hours and Minutes entries +1)
;Dit 20 = Set new divider for chip counter - Low byte only (All entries +1)
;Dit 21 = Set seconds speed up + interval (All entries +1)

;Following fuses are set on ATTiny2313 chip:
;  Divide by 8 
;  Xtl 3-8MHz


.include "tn2313def.inc"

rjmp	START1
rjmp	START1
rjmp	START1
rjmp	START1
rjmp	ClockCntDwnInt		;1 interruption per second - Timer/Counter 1 compare match A


	
START1:				;set up stack pointer and initialize cold start

ldi		r16,low(RAMEND)		; Load low byte address of end of RAM into register R16
out		SPL,R16				; Initialize stack pointer to end of internal RAM

ldi		r16,7		;Set Port A, bits 0-2 for output - no crystal, so port A active
out		DDRA,r16	;Execute data direction with Port A 0-2 as output

clr		r16
out		PortA,r16	;Turn off ports

ldi		r16,126		;Set bits PD1-6 to ones for out, bit PD0 for input
out		DDRD,r16	;Execute data direction with PD0 as input rest as output

clr		r16
out		PortD,r16	;All output ports low - LEDs off
					;No input resistor for input port D,0

ser		R16			;Set all bits for data direction as outputs
out		DDRB,R16	;execute data direction with all as outputs
clr		r16

out		PORTB,R16	;execute to set out ports low

rcall	Delay4seconds 	;to allow voltage to rise 

ldi		r20,50			;set low divider byte at 50
ldi		r21,62			;set high divider byte at 62


in  	r16,TIMSK  		;enable 16-bit match to A register
ori 	r16,64
out 	TIMSK,r16

in 		r16,TCCR1B
ori		r16,11          ; divide by 64 on prescaler
out		TCCR1B,r16      ;enable reset of counter to zero (bit3) after match detection.
                    	;and bits 0,1,2 select prescaling source as 64 of clock

                    	;3,579,545 Xtl with 8 division = 447,443
                    	;  divided by 64 = 6991.3

;  Following is one second timer -- It is set to run slightly fast

ldi  	r16,27          ;comparator high byte for this crystal
out  	OCR1AH,r16
ldi  	r16,79          ;comparator low byte for this crystal 
out  	OCR1AL,r16      ;for one second intervals



;Following puts clock correction bytes in EEPROM over into SRAM

ldi		r16,$00		;EEPROM address
rcall	FetchEEPROM
sts		$88,r16		;put LSD of clock correction interval permanent store low bytes into SRAM permanent store
sts		$8C,r16		;put LSD of clock correction interval variable store into SRAM

ldi		r16,$01		;EEPROM address	
rcall	FetchEEPROM
sts		$89,r16		;put MSD of clock correction interval permanent store low bytes into SRAM permanent store
sts		$8D,r16		;put MSD of clock correction interval variable store into SRAM

ldi		r16,$02		;EEPROM address
rcall	FetchEEPROM
sts		$8A,r16		;put LSD of clock correction interval permanent store high bytes into SRAM permanent store
sts		$8E,r16		;put LSD of clock correction interval permanent store high bytes into SRAM

ldi		r16,$03		;EEPROM address
rcall	FetchEEPROM
sts		$7B,r16		;put MSD of clock correction interval permanent store high bytes into SRAM permanent store
sts		$7F,r16		;put MSD of clock correction interval permanent store high bytes into SRAM 



;INITIALIZE CLOCK to 8:00 a.m. - Sunday

ldi		r16,1		;high byte of 8:00 a.m. time
sts		$69,r16
sts		$6A,r16		;sunday = 1

ldi		r16,224		;low byte of 8:00 a.m. time
sts		$68,r16

clr		r15			;zero seconds counter


;---------------
	
clr		r17			;clear flags registers
clr		r18
clr		r19

sei					;ON interrupts

;-------------------------------------

TTT:				;Start of program loop - red flash first

sbis	PIND,0		;skip if PIND,0 is set - switch is up
rjmp	TTTout		;jump out for command entry

lds		r24,$70		;RED code
lds		r25,$71		;RED present bit position (bit moves from left to right)

and		r25,r24
brne	TTTred
rjmp	TTT1


TTTred:

sbi		PortB,0		;on RED led
rcall	EEDelay40th
cbi		PortB,0		;off RED led

sbis	PIND,0		;skip if switch not down
rjmp	TTTout		;jump out for command entry

rcall	EEDelay10th
sbis	PIND,0
rjmp	TTTout		;jump out for command entry

sbrs	r18,0		;skip if r18,0(1) is set (2nd flash for RED)
					;  When a pill is taken, this flag can be reset
rjmp	TTTe		;quit -- no 2nd flash

sbi		PortB,0		;ON led for 2nd flash
rcall   EEDelay40th
cbi		PortB,0		;OFF led
sbis	PIND,0		;skip if switch not down
rjmp	TTTout		;jump out for command entry


TTTe:

rcall	EEDelay500ms	;Delay interruptible 1/2 second 


TTT1:				;Yellow Flash section

sbis	PIND,0			;skip if button is up
rjmp	TTTout		;jump out for command entry

lds		r24,$73		;Yellow code
lds		r25,$74		;Yellow present position

and		r25,r24
brne	TTTyellow
rjmp	TTT2


TTTyellow:

sbi		PortB,1		;ON yellow led
rcall	EEDelay40th
cbi		PortB,1		;OFF yellow led
sbis	PIND,0		;skip if switch is up
rjmp	TTTout		;jump out for command entry

rcall	EEDelay10th
sbis	PIND,0
rjmp	TTTout		;jump out for command entry

sbrs	r18,1		;skip if r18,1(2) is set (2nd flash needed)
rjmp	TTT1e

sbi		PortB,1		;ON yellow led
rcall	EEDelay40th
cbi		PortB,1		;OFF yellow led
sbis	PIND,0		;skip if switch is up
rjmp	TTTout		;jump out for command entry


TTT1e:

rcall	EEDelay500ms	;Delay interruptible 1/2 second 


TTT2:				;GREEN section

sbis	PIND,0			;skip if PIND,0 is set - switch is up
rjmp	TTTout		;jump out for command entry

lds		r24,$76		;Green LED code
lds		r25,$77		;Green present position

and		r25,r24
brne	TTTgreen
rjmp	TTT3


TTTgreen:

sbi		PortB,2		;On GREEN led
rcall	EEDelay40th
cbi		PortB,2		;Off GREEN led
sbis	PIND,0		;skip if switch is up
rjmp	TTTout		;jump out for command entry

rcall	EEDelay10th
sbis	PIND,0		;skip if switch is up
rjmp	TTTout		;jump out for command entry

sbrs	r18,2		;skip if r18,2(4) is set (2nd flash needed)
rjmp	TTT2e

sbi		PortB,2		;ON green led
rcall	EEDelay40th
cbi		PortB,2		;OFF green led
sbis	PIND,0		;skip if switch is up
rjmp	TTTout		;jump out for command entry


TTT2e:

rcall	EEDelay500ms	;Delay interruptible 1/2 second 


TTT3:				;BLUE section

sbis	PIND,0			;skip if PIND,0 is set - switch is up
rjmp	TTTout		;jump out for command entry

lds		r24,$79
lds		r25,$7A

and		r25,r24
brne	TTTblue
rjmp	TTT				;Go back to loop start


TTTblue:

sbi		PortB,3		;ON blue led
rcall	EEDelay40th
cbi		PortB,3		;OFF blue led
sbis	PIND,0		;skip if switch is up
rjmp	TTTout		;jump out for command entry

rcall	EEDelay10th
sbis	PIND,0		;skip if switch is up
rjmp	TTTout		;jump out for command entry

sbrs	r18,3		;skip if r18,3(8) is set (2nd flash)
rjmp	TTT4		;loop back for another loop

sbi		PortB,3		;ON blueled
rcall	EEDelay40th
cbi		PortB,3		;OFF blue led
sbis	PIND,0		;skip if switch is up
rjmp	TTTout		;jump out for command entry


TTT4:

rcall	EEDelay500ms	;Delay interruptible 1/2 second 

rjmp	TTT			;loop back to continue looping


TTTout:

rcall	Button

rjmp	Decode		;JUMP into Decode section


;------------------------------------

; THIS IS NOT A SUBROUTINE! 

Decode:				;decodes r30 entry, LSB of r30,31 pair

cpi		r26,1		;toggle RED LED 2nd flash flag
brne	DE02

sbrs	r18,0
rjmp	DE

cbr		r18,1		;clear r18,0(1)
rjmp	DEend2		;jump back into loop


DE:

sbr		r18,1		;set r18,0(1)
rjmp	DEend2		;jump back into loop



DE02:

cpi		r26,2		;toggle YELLOW LED 2nd flash flag
brne	DE03

sbrs	r18,1
rjmp	DE1

cbr		r18,2		;clear r18,1(2)
rjmp	DEend2		;jump back into loop


DE1:

sbr		r18,2		;set r18,1(2)
rjmp	DEend2		;jump back into loop



DE03:

cpi		r26,3		;toggle GREEN LED 2nd flash flag r18,2(4)
brne	DE04

sbrs	r18,2
rjmp	DE2

cbr		r18,4		;clear r18,2(4)
rjmp	DEend2		;jump back into loop


DE2:

sbr		r18,4		;set r18,2(4)
rjmp	DEend2		;jump back into loop



DE04:

cpi		r26,4		;toggle BLUE LED 2nd flash flag, r18,3(8)
brne	DE11

sbrs	r18,3	
rjmp	DE3

cbr		r18,8		;clear r18,3(8)
rjmp	DEend2		;jump back into loop


DE3:

sbr		r18,8		;set r18,3(8)
rjmp	DEend2		;jump back into loop



DE11:

cpi		r26,11		;program RED LED
brne	DE12			;Enter up to eight dit-dah entries for pill code

rcall	EnterREDledDATA

rjmp	DEend2		;jump back into loop


DE12:

cpi		r26,12		;program YELLOW LED
brne	DE13		;Enter up to eight dit-dah entries for pill code

rcall	EnterYELLOWledDATA
rjmp	DEend2		;jump back into loop


DE13:	

cpi		r26,13		;program GREEN LED
brne	DE14		;Enter up to eight dit-dah entries for pill code

rcall	EnterGREENledDATA
rjmp	DEend2		;jump back into loop


DE14:

cpi		r26,14		;program BLUE LED
brne	DE16		;Enter up to eight dit-dah entries for pill code

rcall	EnterBLUEledDATA
rjmp	DEend2		;jump back into loop


DE16:				;Clears all pill codes

cpi		r26,16
brne	DE17  

clr		r16
sts		$70,r16
sts		$73,r16
sts		$76,r16
sts		$79,r16

rjmp	DEend2		;jump back into loop


DE17:				;Toggle a flash of white LED every minute (flag)

cpi		r26,17
brne	DE18

sbrs	r19,5		;skip if r19,5(32) is set (flash white led every minute flag)
rjmp	DE17a

cbr		r19,32		;clear r19,5(32) flag (off flash of LED)
rjmp	DEend2		;jump back into loop


DE17a:

sbr		r19,32			;set r19,5(32) flag (on flash of LED)

sbi		PortB,4			;ON white led
rcall	Delay4th
cbi		PortB,4			;OFF white led (long flash when bit is set to on)

rjmp	DEend2			;jump back into loop


DE18:					;All clock entries must be +1.  That is 11:00 p.m. = 23 but enter 24.  Likewise +1 for minutes

cpi		r26,18
brne	DE22

rcall	SetClock
rjmp	DEend2			;jump back into loop


DE22:  					;TWO entries -- seconds to speed up followed by day interval 
						;YOU MUST use +1 number for both entries.  1 = 0 entry (no correction)

cpi		r26,22
brne	DE25

rcall	SetClockCorrectionInterval
rjmp	DEend2			;jump back into loop


DE25:	;Rotates ALL LED masks one position to effect midnight rotation

cpi		r26,25
brne	DEend			;End of commands

rcall	RotateAllLEDmasks

rjmp	DEend2


DEend:

ldi		r16,8			;set up for eight flashes of white LED - (No correct selection was made)
rcall	FastWhiteLEDflash

rjmp	TTT				;jump back into loop


DEend2:

ldi		r16,3
rcall	FastWhiteLEDflash

rjmp	TTT


;--------------------------------------

ClockCntDwnInt: 		;interrupt to update clock counters

;						;r15 is seconds counter

push	r16
push	r28
push	r29
push	r30
push	r31

in		r16,SREG	;save flags
sts		$0060,r16	;store in SRAM $60

cbr		r19,64		;zero one minute flag -- flag is set for only one second period (val 64)
					


;CLOCK CORRECTION section -- check occurs every new second.  At 15 seconds in, one second is added if flag r18,7(128) flag set.

;  EEPROM - $00,01,02,03 is permanent storage of clock correction interval in EEPROM

;  SRAM $88,89,8A,8B = permanent store of clock correction interval
;  SRAM $8c,d,e,f = variable store of clock correction interval

;  One second per interval is added to the second counter when variable interval reaches zero. (speeds up clock by one second)
;    Thus, length of interval is determined by number of seconds per day according to:  86,400/number of seconds/day desired
;    There are 86,400 seconds per day.   

; Clock Correction Flag Set section

Increment:

sbrc	r18,6			;skip if r18,6(64) is clear  --  0 = clock correction ON)
rjmp	AddIntervalEnd

lds		r28,$8C
lds		r29,$8D
lds		r30,$8E			
lds		r31,$8F			;get variable clock correction interval counter data

sbiw	r28,1			;decrement count by one
sts		$8C,r28
sts		$8D,r29			;store away new count
brcc	Add2

sbiw	r30,1			;decrement upper MS pair by one
sts		$8E,r30
sts		$8F,r31			;store away new count


Add2:

mov		r16,r28
or		r16,r29
or		r16,r30
or		r16,r31			;check for zero count
breq	Add1

rjmp	AddEnd


Add1:					;Reset counter to new count and set r18,7(128) add second next time flag

sbr		r18,128			;set r18,7(128), add one second next time flag

lds		r16,$88
sts		$8C,r16

lds		r16,$89
sts		$8D,r16

lds		r16,$8A
sts		$8E,r16

lds		r16,$8B
sts		$8F,r16			;put new count in clock correction interval counter from permanent count store


AddEnd:


; END of Clock Correction flag set section --  START of clock correction when flag is set and 15 seconds in
; Following section will check for 15 secoond time and add one second if intervalt flag r18,7(128) is set

sbrs	r18,7			;skip if r18,7(128) flag is set - addition of one second is needed flag
rjmp	AddIntervalEnd

mov		r16,r15
cpi		r16,15			;Check if 15 seconds into minute			
brne	AddIntervalEnd

cbr		r18,128			;clear r18,7(128) one second addition flag
inc		r15				;ADD one second to r15, seconds counter


AddIntervalEnd:

;End of clock correction area


CC1b:

inc		r15			;step one count of SECONDS counter


CC1bb:

mov		r16,r15
cpi		r16,60
breq	CCDI1

rjmp	CCDIExit		;return since no action required


CCDI1:				;NEW MINUTE started

sbrs	r19,5		;skip if r19,5(32) flag is set - white flash every minute flag
rjmp	CCDIflash1

sbi		PortB,4		;on WHITE led - flash led because White Flash Every Minute flag is set
rcall	Delay40th
cbi		PortB,4		;off WHITE led


CCDIflash1:

clr		r15			;zero r15 -- SECONDS counter
sbr		r19,64		;set R19 bit 6(64) -- the NEW ONE MINUTE flag


CCDI1Cntr:

lds		r16,$0068		;load low minutes byte into R16

inc		r16
sts		$0068,r16

brne	CCDI1a		;jump out if not zero (no overflow)

lds		r16,$0069		;high byte minutes
inc		r16			;increment high byte minutes
sts		$0069,r16


CCDI1a:

lds		r16,$0069		;check high byte if it is 5 (1440=  5 A0)
cpi		r16,5

breq	CCDI1b

rjmp	CCDIExit


CCDI1b:			

lds		r16,$0068
cpi		r16,160		;1440 minutes reached -- it's now midnight

breq	CCDI1c

rjmp	CCDIExit


CCDI1c:			;zero high and low bytes - it is midnight

clr		r16
sts		$0068,r16		;store away new start at zero of 1440 count
sts		$0069,r16


CCDI1e:					;MIDNIGHT routines



;THE FOLLOWING rolls LED mask bit positions one to left at midnight.   Also, resets position
;  to home position when it exceeds last bit position.

rcall	RotateAllLEDmasks


CCDIExit:		;Exit this interrupt routine

lds		r16,$0060		;SREG temporary store
out		SREG,r16	;restore SREG data

pop		r31
pop		r30
pop		r29
pop		r28
pop		r16

reti				;master unit exit


;-------------------------------------------------------

RotateAllLEDmasks:

sbr		r18,15			;Set bits 0,1,2,3 in r18 (RYGB).  This sets the 2nd flash flags on 
						;  these flags are reset after a "pill is taken" which then eliminates 2nd flash

;Check RED position and rotate or reset

lds		r16,$71			;load  r16 with mask
sbrs	r16,0			;skip if r16,0(1) is set (home position)
rjmp	ROLred1

lds		r16,$72			;load with max location
rjmp	ROLred2


ROLred1:

lsr		r16


ROLred2:

sts		$71,r16			;store new RED mask


;Check YELLOW position and rotate if necessary

lds		r16,$74			;load  r16 with mask
sbrs	r16,0			;skip if r16,0(1) is set (home position)
rjmp	ROLyellow1

lds		r16,$75			;load with max location
rjmp	ROLyellow2


ROLyellow1:

lsr		r16


ROLyellow2:

sts		$74,r16			;store away new Yellow mask


;Check GREEN position and rotate if necessary

lds		r16,$77			;load  r16 with mask
sbrs	r16,0			;skip if r16,0(1) is set (home position)
rjmp	ROLgreen1

lds		r16,$78			;load with max location
rjmp	ROLgreen2


ROLgreen1:

lsr		r16


ROLgreen2:

sts		$77,r16			;store away new Green mask


;Check YELLOW position and rotate if necessary

lds		r16,$7A			;load  r16 with mask
sbrs	r16,0			;skip if r16,0(1) is set (home position)
rjmp	ROLblue1

lds		r16,$7B			;load with max location
rjmp	ROLblue2


ROLblue1:

lsr		r16


ROLblue2:

sts		$7A,r16			;store away new Blue mask


ret 


;--------------------------------------------------------

SetClock:	;sets clock for AM/PM, hour, minutes, and day-of-week
			;  24 hour time entry required
			;1440 = midnight = 0

;SPECIAL NOTE:  -- Unit Number, Hours, and Minutes are 
;  done by dit (1), dah(10), and hold down (50) for numbers to be entered.
;  a RED flash precedes all entries that use the dit/dah/hold down button push.

push	r1		;minutes accumulator
push	r28
push	r29		;accumulator for minutes -- Y register pair

rcall	ButtonFlashStall	;White LED flashes awaiting entry

lds		r16,$7F		;get low total count byte from SRAM
dec		r16			;one less than entry so ZERO entry is made possible
					;  1 = 0,  2 = 1,  3 = 2,  etc.

clr		r28
clr		r29			;zero register pair, r28,29


SC1a:

tst		r16
breq	SC1b

adiw	r28,60		;add 60 to change hours into minutes
dec		r16
brne	SC1a


SC1b:

rcall	ButtonFlashStall	;White LED flashes awaiting entry

lds		r16,$7F			;get LSB of total count
dec		r16				;one less to make for zero entry
						;  1 = 0,  2 = 1,  3 = 2,  etc.
add		r28,r16			;add r16 to r30
brcc	SetEnd

inc		r29				;add one to r31 (carry)

;minute total now complete


SetEnd:

cli				;turn off global interrupts

sts		$0068,r28		;store low byte in $0068
sts		$0069,r29		;store high byte in $0069
clr		r15				;zero seconds counter


sei						;turn global interrupts back on

pop		r29
pop		r28
pop		r1

ret


;-----------------------------------

WriteEEPROM:		;write new code into EEPROM
					;r16 has data
					;r30 has memory location 

cli					;OFF interrupts

WEE:

sbic	EECR,EEPE	;wait until EEPROM ready for new entry
rjmp	WEE

out		EEAR,r30	;set up address in address register
out		EEDR,r16	;put contents of r16 into EEPROM data location

sbi		EECR,EEMPE	;write logical one to EEMPE

sbi		EECR,EEPE	;start eeprom write by setting EEPE

sei					;ON interrupts

ret


;--------------------------------------------------

FetchEEPROM:		;grabs data in EEPROM, r30 address location
					;r16 will contain data byte

FEE:

sbic	EECR,EEWE	;Wait for completion of previous write
rjmp	FEE

out		EEAR,r30	;set up address in address register
sbi		EECR,EERE	;start EEPROM read by writing EERE

in		r16,EEDR	;read data from data register

ret					;code data now in r5-9


;-------------------------------------------

ReadInClockCorrectionSecondsInterval:	;from EEPROM) read in seconds and interval for clock correction
										;$03 and $04 are second and interval stored in EEPROM	

push	r30
push	r31

ldi		r30,$02					;point to first storage location, seconds
clr		r31

rcall	FetchEEPROM
sts		$0064,r16				;seconds

inc		r30						;increment to next location, interval
rcall	FetchEEPROM
sts		$0065,r16				;interval

pop		r31
pop		r30

ret


;------------------------

ButtonStall:		;Use this entrance when you wish to wait for a button push.
					;  It goes to Button when switch goes down.
					;NOTE:  This coding same as used for OBH2a 6/27/2010 with modifications for i/o ports

clr		r16
out		PortB,r16	;OFF all LEDs

sbis	PIND,0		;skip if switch is up
rjmp	Button

rjmp	ButtonStall


;-------------------

ButtonFlashStall:	;Flash white LED every third of second until Button is pushed

sbi		PortB,4		;ON white LED
rcall	EEDelay40th
cbi		PortB,4

sbis	PIND,0		;skip if switch is up
rjmp	Button

rcall	EEDelay300ms
rjmp	ButtonFlashStall


;-------------------

Button:		;Takes in dit, dah, holddown, and coded input

clr		r16
out		PortB,r16	;OFF all LEDs

push	r0		;number of code digits
push	r1		;low code byte
push	r2		;next high code byte
push	r3		;next high code byte
push	r4		;highest code byte
push	r18		;dit count
push	r20		;holddown count
push	r21		;Max code count via shift right of bit
push	r23		;universal counter
push	r28		;dah count
push	r30		;has total count of entry in Z register, r30,31
push	r31

;r30,r31 (Z register pair contain total value of entry)

clr		r18		;Dit Count
clr		r20		;HoldDown count
clr		r21		;Max code via shift right of bit
clr		r23		;universal counter
clr		r28		;dah count
clr		r0		;clear number of bits in code
clr		r1		;clear lowest code count
clr		r2		;clear next code count
clr		r3		;clear next code count
clr		r4		;clear next code count (clear all code counts)


				;r30,31 has total count

B1:

inc		r23			;universal counter
rcall	Delay40th
cpi		r23,60		;delay before first holddown count - 40=one second 
brsh	B3			;branch if - it is a holddown count

sbis	PIND,0		;skip if switch is NOT down
rjmp	B1


B2:

cpi		r23,20		;compare r23 for 20 (1/2 second dah count)
brlo	B2b			;branch if lower (dit count)


B2a:				;it is dah count

rcall	MaxCountInR21

inc		r28			;increment dah count

sec					;set carry flag -- it is dah entry
rol		r1
rol		r2			;rotate data into registers r1-r4
rol		r3
rol		r4

sbi		PortB,4		;on white LED
rcall	Delay40th
rcall	Delay40th	;delay for longer on of LEDs
rcall	Delay40th
rcall	Delay40th
cbi		PortB,4		;off LED
clr		r23
inc		r0			;increment number of code digits


B2d:				;no touch/switch up detection

sbis	PIND,0		;skip if switch is up
rjmp	B2e			;jump back - there is a new entry coming in

inc		r23
rcall	Delay40th
cpi		r23,85		;has switch been up 85 counts? 40=one second (2 + seconds)
brsh	B4Decode1
rjmp	B2d


BPEnd1:

rjmp	BPEnd


B2e:

clr		r23
rjmp	B1


B2b:				;Dit count detected

rcall	MaxCountInR21

inc		r18			;increment dit count

clc					;clear carry flag -- it is dit entry
rol		r1
rol		r2			;rotate data into registers r1-r4
rol		r3
rol		r4

sbi		PortB,4
rcall	Delay40th
cbi		PortB,4

inc		r0			;increment number of code digits (r0)
clr		r23			;clear r23
rjmp	B2d


B3:					;Holddown count has been detected

inc		r20			;increment holddown count
sbi		PortB,4
rcall	Delay40th
rcall	Delay40th
cbi		PortB,4		;flash RED LED

clr		r23			;reset r23 to zero count


B4:

inc		r23
rcall	Delay40th
sbic	PIND,0		;skip if switch is down
rjmp	B4a			;jump out and look for next entry

cpi		r23,30		;pause between continuous hold down of switch
brlo	B4

rjmp	B3


B4a:

clr		r23
rjmp	B2d			;Check for exit or new entry



B4Decode1:

;This section finalizes.  It combines counts

clr		r30
clr		r31			;ready registers for input

mov		r16,r28		;move dah count in r28 into r16


B4comb1:

tst		r16
breq	B4comb2

adiw 	r30,10		;add value of 10 in r30 to r314
dec		r16			
rjmp	B4comb1


B4comb2:

add		r30,r18		;add value in r18 (dit count)
brcs	B4comb2a
rjmp	Bholdown1


B4comb2a:

inc		r31			;carry was set so increment higher byte


Bholdown1:

;Combined value now in Z register pair, r30,31

mov		r16,r20		;put hold down count into r16


Bholdown1b:

tst		r16			;test r20(r16), hold down count
breq	BPEnd
adiw	r30,50		;add fifty count to total
dec		r16
rjmp	Bholdown1b

;Complete combined value now in Z register pair


BPEnd:

;Store away in SRAM $00E7-00EB, Dit,Dah,HolddownCount,LSD Total, MSD Total

sts		$7C,r18		;number of dits
sts		$7D,r28		;number of dahs
sts		$7E,r20		;number of hold down counts
sts		$7F,r30		;LSD of total count
sts		$80,r31		;MSD of total count
sts		$81,r1		;Code (8 digits only)
sts		$82,r21		;Max Location.  Combined dit dah shifts of dit dah count
						; (Example: 5 shifts = 00010000)-Max code count 

mov		r26,r30
mov		r27,r31		;put LSD and MSD of total count into r26,27


pop		r31
pop		r30
pop		r28
pop		r23
pop		r21
pop		r20
pop		r18
pop		r4
pop		r3
pop		r2
pop		r1
pop		r0

lds		r26,$7F		;put low byte (total count) into r26
lds		r27,$80		;put high byte (total count) into r27

ret


;-------------------------------

MaxCountInR21:

tst		r21			;check to see if this is first entry for string of dit/dah entries
breq	B2p1		;This section rotates a bit left for each dit/dah entry (max bit position entry)

rjmp	B2p2


B2p1:				;This is first iteration of new entry

ldi		r21,1		;Set first right bit on
rjmp	B2p3


B2p2:				;all other entries will rotate bit one place to left

lsl		r21			;logical right shift of bit in r21 (max number of bit entries)


B2p3:	ret


;------------------

;EEDelay Routines --- EXIT EARLY -- Quickly exits upon Button Push

;------------------

EEDelay800th:		;BASE delay for other early exit delays

push	r16
ldi		r16,109		;for 3.579545 xtl /8


EED800a:

sbis	PIND,0		;jump out of delay if switch goes down (low)
rjmp	EED800end

dec		r16
brne	EED800a


EED800end:

pop	r16

ret


;---------------

EEDelay40th:

push	r16
ldi		r16,20


EEDelay40a:

sbis	PIND,0		;jump out of delay if switch goes down (low)
rjmp	EEDelay40end

rcall 	Delay800th
dec		r16
brne	EEDelay40a


EEDelay40end:

pop		r16

ret


;--------------

EEDelay10th:

push	r16
ldi		r16,80


EEDelay10a:

sbis	PIND,0		;jumpe out if switch down
rjmp	EEDelay10end

rcall	EEDelay800th
dec		r16
brne	EEDelay10a


EEDelay10end:

pop		r16
ret

;---------------

EEDelay300ms:

push	r16
ldi		r16,3


EEDelay300a:

sbis	PIND,0		;skip if switch down
rjmp	EEDelay300end

rcall	EEDelay10th
dec		r16
brne	EEDelay300a


EEDelay300end:

pop		r16

ret


;---------------

EEDelay500ms:

push	r16
ldi		r16,5


EEDelay500a:

sbis	PIND,0		;skip if switch down
rjmp	EEDelay500end

rcall	EEDelay10th
dec		r16
brne	EEDelay500a


EEDelay500end:

pop		r16

ret


;---------------

EEDelay1sec:

push	r16
ldi		r16,10


EEDelay1sec1:

sbis	PIND,0		;jump out if switch down
rjmp	EEDelay1end

rcall	EEDelay10th
dec		r16
brne	EEDelay1sec1


EEDelay1end:

pop		r16

ret


;---------------------------------------

; DELAY ROUTINES

;---------------------------------------

Delay800th:			;BASE delay for other delays

push 	r16
ldi		r16,109		;for 3.579MHz crystal/ 8


D800a:

dec		r16
nop
nop
brne	D800a

pop		r16

ret


;-------------------------------------

Delay40th:

push	r16
ldi		r16,20


D40a:

rcall 	Delay800th
dec		r16
brne	D40a

pop		r16

ret


;-----------------------------------------

Delay10th:			;1/10th second delay

push	r16
ldi		r16,80

rjmp	D40a


;-----------------------------------------

Delay4th:			;1/4 second delay

push	r16
ldi		r16,200

rjmp	D40a


;------------------------------------------

Delay4seconds:

push	r16
ldi		r16,40		;for 4 second delay


D4a:

rcall	Delay10th
dec		r16
brne	D4a

pop		r16

ret


;------------------------------------

FastWhiteLEDflash:		;r16 has number of flashes

sbi		PortB,4		;on white LED
rcall	Delay40th
cbi		PortB,4		;off white LED

rcall	Delay40th

dec		r16
brne	FastWhiteLEDflash

ret


;---------------------------------------

WhiteLEDflash:		;r16 contains number of flashes


sbi		PortB,4		;on White LED
rcall	Delay40th
cbi		PortB,4		;off LED

rcall	Delay4th	;1/4 second delay
rcall	Delay4th	;totsl 1/2 second delay


dec		r16
brne	WhiteLEDflash

ret


;-------------------------------------

EnterREDledDATA:

rcall	ButtonFlashStall	;White LED flashes awaiting entry

lds		r16,$81				;load RED code from Button routine
sts		$70,r16

lds		r16,$82
sts		$72,r16				;put away max code location for RED
sts		$71,r16			;store in RED present bit position (mask)

sbr		r18,1			;set r18,0(1) 2nd flash flag - RED

ret


;--------------------------------------

EnterYELLOWledDATA:

rcall	ButtonFlashStall	;White LED flashes awaiting entry

lds		r16,$81			;load YELLOW code
sts		$73,r16

lds		r16,$82
sts		$75,r16			;put away max code location for YELLOW
sts		$74,r16			;load mask

sbr		r18,2			;set r18,1(2), YELLOW 2nd flash flag

ret


;-------------------------------------

EnterGREENledDATA:

rcall	ButtonFlashStall	;White LED flashes awaiting entry

lds		r16,$81
sts		$76,r16			;put away GREEN code

lds		r16,$82
sts		$78,r16			;put away mas code location for GREEN
sts		$77,r16			;put away start position far left (mask)

sbr		r18,4			;set r18,2(4), Green 2nd flash flag

ret


;------------------------------------

EnterBLUEledDATA:

rcall	ButtonFlashStall	;White LED flashes awaiting entry

lds		r16,$81
sts		$79,r16			;put away BLUE code

lds		r16,$82
sts		$7B,r16			;put away BLUE maximum location
sts		$7A,r16			;put away start position (mask)

sbr		r18,8			;on 2nd flash flag - r18,3(8) - BLUE

ret


;---------------------------------------

FlashWhiteLED:		;r16 contains number of flashes

FWa:

sbi		PortB,4		;ON white led
rcall	Delay40th
cbi		PortB,4		;OFF white led

rcall	Delay4th
rcall	Delay4th
dec		r16
brne	FWa

ret


;-------------------------------------------

FlashOutDataInR16:		;Data placed in r16 is flashed out with double flash = 1 (low to high)

push	r17				;scratch
ldi		r17,8

rcall	Delay4seconds


FOD1a:

sbrc	r16,0			;skip if zero bit in r16 is clear
rjmp	FOD1

sbi		PortB,4			;ON white LED
rcall	Delay40th
cbi		PortB,4			;OFF LED

rcall	Delay4th
rcall	Delay4th
rcall	Delay4th

rjmp	FOD2


FOD1:

sbi		PortB,4
rcall	Delay40th
cbi		PortB,4

rcall	Delay40th
rcall	Delay40th
rcall	Delay40th

sbi		PortB,4
rcall	Delay40th
cbi		PortB,4

rcall	Delay4th
rcall	Delay4th
rcall	Delay4th


FOD2:

dec		r17
breq	FODend

lsr		r16				;shift r16 right on digit to read next highest bit (low to high)
rjmp	FOD1a


FODend:

pop		r17
rcall	Delay4seconds

ret


;----------------------------------

LongFlash: 

sbi		PortB,4		;ON white LED
rcall	Delay4th
rcall	Delay4th
rcall	Delay4th
rcall	Delay4th
cbi		PortB,4		;OFF white LED

ret


;----------------------------------

SetClockCorrectionInterval:

; This section sets a one second speed up of clock every so many seconds.  To determine the entry put in,
;   divide the total number of seconds per day 86,400 by the number (and fraction) of seconds per day
;   you wish to speed up the clock.  Example, to speed it up 5.73 seoncs per day you would divide 86,400
;	by 5.73, yielding the number -- 15,079.  This is the entry number.  It means that every 15,079 second
;   count the micro-controller will add an extra second.

; To enter, use the first entry for the THOUSANDS unit.  You would enter 15.  But because the PLUG ONE 
;   system is used, you would enter one plus 15 or 16.

; After entry, wait for flash of LED to show acceptance of your first entry, before entering second entry.

; For the second entry, you would then use the hold down count of 50 and dah count of 7 (units of ten) and
;   a dit count of 9.  BUT, because of the Plus one system you would enter 80 instead of 79 or 8 dah counts.


clr		r28
clr		r29
clr		r30
clr		r31				;ZERO counter pairs

sbr		r18,64			;set r18,6(64), clock correction.  1 = Clock correction OFF   0 = clock correction ON

ldi		r16,2			;give two flashes of Blue LED - two flashes indicate 2 entries needed
rcall	FlashWhiteLED

;THOUSANDS entry -- DIT = ZERO --- One plus entry system used so for 18,000, enter 19

rcall	ButtonStall		;THOUSANDS entry (HoldDown=50, Dah=10, Dit=1)(One Plus entry system for LSD, the Dit)
						;  r26,27 have low and high bytes of Button Count

sbiw	r26,1				;it is a PLUS ONE entry so, one less

mov		r16,r26
or		r16,r27			;check for zero

breq	SCC10


SCC1:					;Thousands Section - add one thousand for each value in r26

mov		r24,r26
mov		r25,r27			;save 1000 entry values in r24,25

cbr		r18,64			;clear clock correction flag, r18,6(64), 0 = ON clock correction


SCC10:					;Enter remaining group after the 1,000 group

ldi		r16,1			;give one flash - last entry
rcall	FlashWhiteLED

rcall	ButtonStall		;r26,27 have value put in by touch plate
sbiw	r26,1			;reduce by one - Plus One Entry system

mov		r16,r26
or		r16,r27
breq    SCC10c			;bypass setting of flag r18,6(64)

cbr		r18,64			;clear r18,6(64) ON clock correction flag


SCC10c:

mov		r28,r26
mov		r29,r27			;put low keyed in values into low bytes of counter r28,29,30,31


SCC10a:					;Add 1000 to r28-31 registers for amount in r24,25

mov		r16,r24
or		r16,r25			;check for zero entry
breq	SCCEnd

rcall	Add1000
						
sbiw	r24,1			;decrement by one, the r24,25 pair (1000 entry data saved earlier)
rjmp	SCC10a


SCCEnd:

sts		$88,r28
sts		$8C,r28
sts		$89,r29
sts		$8D,r29
sts		$8A,r30
sts		$8E,r30
sts		$8B,r31
sts		$8F,r31		;store away new clock correction intervals in SRAM

ldi		r17,$00		;EEPROM address
mov		r16,r28
rcall	WriteEEPROM

ldi		r17,$01		;EEPROM address	
mov		r16,r29
rcall	WriteEEPROM

ldi		r17,$02		;EEPROM address
mov		r16,r30
rcall	WriteEEPROM

ldi		r17,$03		;EEPROM address
mov		r16,r31
rcall	WriteEEPROM

ret


;------------------------------------------

Add1000:			;Adds 1000 to r28-31 registers each time subroutine called

ldi		r16,31


Add1000a:

adiw	r28,32		;Add 32 31 times = 992
brcc	Add1000b

adiw	r30,1			;account for carry


Add1000b:

dec		r16
brne	Add1000a

adiw	r28,8			;add remainder (8) (992+8=1000)
brcc	Add1000Exit

adiw	r30,1			;account for carry


Add1000Exit:

ret


;--------------------------------------------
